@@ -54,7 +54,7 @@ struct nbd_device {
 
 	struct mutex tx_lock;
 	struct gendisk *disk;
-	int blksize;
+	loff_t blksize;
 	loff_t bytesize;
 	int xmit_timeout;
 	bool disconnect; /* a disconnect has been requested by user */
@@ -135,7 +135,7 @@ static void nbd_size_update(struct nbd_device *nbd, struct block_device *bdev)
 }
 
 static int nbd_size_set(struct nbd_device *nbd, struct block_device *bdev,
-			int blocksize, int nr_blocks)
+			loff_t blocksize, loff_t nr_blocks)
 {
 	int ret;
 
@@ -144,7 +144,7 @@ static int nbd_size_set(struct nbd_device *nbd, struct block_device *bdev,
 		return ret;
 
 	nbd->blksize = blocksize;
-	nbd->bytesize = (loff_t)blocksize * (loff_t)nr_blocks;
+	nbd->bytesize = blocksize * nr_blocks;
 
 	nbd_size_update(nbd, bdev);
 
@@ -750,7 +750,7 @@ static int __nbd_ioctl(struct block_device *bdev, struct nbd_device *nbd,
 
 	case NBD_SET_SIZE:
 		return nbd_size_set(nbd, bdev, nbd->blksize,
-				    arg / nbd->blksize);
+					div_s64(arg, nbd->blksize));
 
 	case NBD_SET_SIZE_BLOCKS:
 		return nbd_size_set(nbd, bdev, nbd->blksize, arg);
@@ -959,7 +959,7 @@ static int nbd_dev_dbg_init(struct nbd_device *nbd)
 		return PTR_ERR(f);
 	}
 
-	f = debugfs_create_u32("blocksize", 0444, dir, &nbd->blksize);
+	f = debugfs_create_u64("blocksize", 0444, dir, &nbd->blksize);
 	if (IS_ERR_OR_NULL(f)) {
 		dev_err(nbd_to_dev(nbd), "Failed to create debugfs file 'blocksize', %ld\n",
 			PTR_ERR(f));
